#! /bin/python
import pkg_resources
import logging
from docopt import docopt
from dela.ListCommand import ListCommand
from dela.logger import log

doc = """dela

CLI to list todos in markdown files, like Obsidian Vaults.

Usage:
  dela -h | --help
  dela --version
  dela list [-v] [--all] [--today] [--done] [--someday] [--status=<symbol>] [--tag=<string>...] [--sort_by=<key>] [--format=<string>] [<glob>]

Options:
  -h --help                     Show this screen
  -v --verbose                  Enable logging
  -a --all                      Show all todos including closed ones
  -t --today                    Show only tasks due today or earlier
  -d --done                     Show only done tasks
  --someday                     Show someday tasks marked with [s] status
  -s --status=<symbol>          Filter by status (x, a, c, ~, ...)
  -tag=<string>                 Filter by tag (#work, #home, etc)
  --sort_by=<key>               Sort by given key
  --format=<string>             Format result with given template string.
  --version                     Show version.


Template example:
    dela list --format='- [$status] $file: $title'

    NOTE: Unicode colors are supported


Template and Sorting Keys:
    - message
    - date
    - status
    - tags
    - file

    NOTE: In templates variable should be prefixed with the $ sign.


Supported todo examples:
    - [ ] Todo
    - [x] Done
    - [n] Next
    - [~] Pending
    - [c] Cancelled
    - [ ] 20230703 Todo with a date. If it's the day, `dela list -t` will list this guy
    - [ ] Todo with tags #tagone #tagtwo
    - [ ] 23330101 Future task will be hidden until the day become
"""

version = pkg_resources.get_distribution("dela").version

def main():
    args = docopt(doc, version=version)

    if args['--verbose']:
        log.setLevel(logging.INFO)
        log.info('Verbose logging has been enabled')

    log.info(f'{args}')

    if args['list'] == True:
        ListCommand(args).run()
        exit(0)


if __name__ == '__main__':
    main()



import os, glob


class FileReader:
    @staticmethod
    def get_files(g):
        return [file for file in glob.glob(os.path.expanduser(g))]

    @staticmethod
    def read_file(path):
        with open(path, 'r') as f:
            return f.read().splitlines(True)



import operator
from datetime import datetime
from dela.TodoPresentation import TodoPresentation
from dela.logger import log
from dela.FileReader import FileReader
from dela.Todo import Todo


class ListCommandConfig(object):
    def __init__(self, args):
        self.glob = args['<glob>'] if args['<glob>'] else '*.md'
        self.format = (
            args['--format']
            if args['--format']
            else '\u001b[30m- \u001b[0m\u001b[01m[$status]\u001b[0m \u001b[31m$file:\u001b[0m $title \u001b[0m\u001b[34m$tags\u001b[0m \u001b[31m$date\u001b[0m'
        )
        self.filter_by_status = args['--status'] if args['--status'] else None
        self.show_all = True if args['--all'] else False
        self.sort_by = args['--sort_by'] if args['--sort_by'] else None
        self.only_today = True if args['--today'] else False
        self.only_done = True if args['--done'] else False
        self.only_someday = True if args['--someday'] else False
        self.filter_by_tags = args['--tag'] if args['--tag'] else None

    def __str__(self):
        return str(self.__class__) + ': ' + str(self.__dict__)


class ListCommand:
    def __init__(self, args) -> None:
        self.config = ListCommandConfig(args)

    def run(self):
        log.info(f'Execute list command with config: {self.config}')

        files = FileReader.get_files(self.config.glob)
        log.debug(f'Match files: {files}')

        result = []
        for file_path in files:
            log.debug(f'Parsing file: {file_path}')

            local = []
            for line in FileReader.read_file(file_path):
                todo = Todo.from_line(line, file_path)
                if todo:
                    local.append(todo)

            if len(local):
                log.info(f'Parsed file: {file_path}')
                log.info(f'Found {len(local)} todo(s)')
                log.info(f'Todos: {local}')
            result += local

        result = self.filter(result)
        result = self.sort(result)

        presentation = TodoPresentation(self.config.format)
        for i in result:
            presentation.present(i)

    def filter(self, todos):
        result = todos

        YYYYmmDD = int(datetime.now().strftime('%Y%m%d'))

        if not self.config.show_all and not self.config.only_done and not self.config.only_someday:
            result = [
                i
                for i in result
                if i.status
                not in [
                    *Todo.STATUSES_DONE,
                    *Todo.STATUSES_ARCHIVED,
                    *Todo.STATUSES_CLOSED,
                    *Todo.STATUSES_SOMEDAY,
                ] 
                and (not bool(i.date) or int(i.date) < YYYYmmDD)
            ]

        if self.config.only_someday:
            result = [i for i in result if i.status in Todo.STATUSES_SOMEDAY]

        if self.config.only_done:
            result = [i for i in result if i.status not in Todo.STATUSES_DONE]

        if self.config.filter_by_status is not None:
            result = [
                i for i in result if i.status == self.config.filter_by_status
            ]

        if self.config.filter_by_tags:
            result = [
                i for i in result if bool(set(i.tags) & set(self.config.filter_by_tags))
            ]

        if self.config.only_today:
            result = [i for i in result if bool(i.date) and int(i.date) < YYYYmmDD]

        return result

    def sort(self, todos):
        result = todos

        if self.config.sort_by:
            result = sorted(
                todos,
                reverse=True,
                key=lambda x: getattr(x, self.config.sort_by) # type: ignore
            )

        return result



import re

TODO_RE = '^\s*-\s+\[(?P<status>.{1})\]\s+(?:(?P<date>\d{8})\s+)?(?P<title>.*?)(?:\s+(?P<tagline>#.*))?$'


class Todo(object):
    STATUSES_DONE = ['x', 'X']
    STATUSES_ARCHIVED = ['a', 'A']
    STATUSES_CLOSED = ['c', 'C', '-']
    STATUSES_SOMEDAY = ['s', 'S']
    STATUSES_IN_PROGRESS = ['~']

    title = None
    date = None
    file = None
    tags = []
    status = None

    def __init__(self, title, status, tags, file, date):
        self.title = title
        self.date = date
        self.status = status
        self.tags = tags
        self.file = file

    @staticmethod
    def from_line(line, file):
        """
        format: - [ ] [YYYYmmDD] Todo #tag #tag
        """
        match = match = re.search(TODO_RE, line)

        if not match:
            return None

        gd = match.groupdict()

        return Todo(
            title=gd['title'],
            status=gd['status'],
            date=gd['date'] if 'date' in gd and gd['date'] else '',
            tags=gd['tagline'].split(' ')
            if 'tagline' in gd and gd['tagline'] is not None
            else [],
            file=file,
        )

    def __repr__(self):
        return f'title: {self.title}\ndate: {self.date}\ntags: {",".join(self.tags)}\nstatus: {self.status}\nfile: {self.file}'
